@@ -24,7 +24,6 @@ from sklearn.metrics import mean_squared_error
 from sklearn.model_selection import train_test_split
 from sklearn.utils import check_random_state, tosequence
 from sklearn.utils._mocking import NoSampleWeightWrapper
-from sklearn.utils._testing import assert_almost_equal
 from sklearn.utils._testing import assert_array_almost_equal
 from sklearn.utils._testing import assert_array_equal
 from sklearn.utils._testing import skip_if_32bit
@@ -46,7 +45,7 @@ true_result = [-1, 1, 1]
 
 # also make regression dataset
 X_reg, y_reg = make_regression(
-    n_samples=500, n_features=10, n_informative=8, noise=10, random_state=7
+    n_samples=100, n_features=4, n_informative=8, noise=10, random_state=7
 )
 y_reg = scale(y_reg)
 
@@ -59,7 +58,7 @@ iris.data = iris.data[perm]
 iris.target = iris.target[perm]
 
 
-@pytest.mark.parametrize("loss", ("deviance", "exponential"))
+@pytest.mark.parametrize("loss", ("log_loss", "exponential"))
 def test_classification_toy(loss):
     # Check classification on a toy dataset.
     clf = GradientBoostingClassifier(loss=loss, n_estimators=10, random_state=1)
@@ -71,40 +70,147 @@ def test_classification_toy(loss):
     assert_array_equal(clf.predict(T), true_result)
     assert 10 == len(clf.estimators_)
 
-    deviance_decrease = clf.train_score_[:-1] - clf.train_score_[1:]
-    assert np.any(deviance_decrease >= 0.0)
+    log_loss_decrease = clf.train_score_[:-1] - clf.train_score_[1:]
+    assert np.any(log_loss_decrease >= 0.0)
 
     leaves = clf.apply(X)
     assert leaves.shape == (6, 10, 1)
 
 
 @pytest.mark.parametrize(
-    "params, err_msg",
+    "params, err_type, err_msg",
     [
-        ({"n_estimators": 0}, "n_estimators must be greater than 0"),
-        ({"n_estimators": -1}, "n_estimators must be greater than 0"),
-        ({"learning_rate": 0}, "learning_rate must be greater than 0"),
-        ({"learning_rate": -1.0}, "learning_rate must be greater than 0"),
-        ({"loss": "foobar"}, "Loss 'foobar' not supported"),
-        ({"min_samples_split": 0.0}, "min_samples_split must be an integer"),
-        ({"min_samples_split": -1.0}, "min_samples_split must be an integer"),
-        ({"min_samples_split": 1.1}, "min_samples_split must be an integer"),
-        ({"min_samples_leaf": 0}, "min_samples_leaf must be at least 1 or"),
-        ({"min_samples_leaf": -1.0}, "min_samples_leaf must be at least 1 or"),
-        ({"min_weight_fraction_leaf": -1.0}, "min_weight_fraction_leaf must in"),
-        ({"min_weight_fraction_leaf": 0.6}, "min_weight_fraction_leaf must in"),
-        ({"subsample": 0.0}, r"subsample must be in \(0,1\]"),
-        ({"subsample": 1.1}, r"subsample must be in \(0,1\]"),
-        ({"subsample": -0.1}, r"subsample must be in \(0,1\]"),
-        ({"max_depth": -0.1}, "max_depth must be greater than zero"),
-        ({"max_depth": 0}, "max_depth must be greater than zero"),
-        ({"init": {}}, "The init parameter must be an estimator or 'zero'"),
-        ({"max_features": "invalid"}, "Invalid value for max_features:"),
-        ({"max_features": 0}, r"max_features must be in \(0, n_features\]"),
-        ({"max_features": 100}, r"max_features must be in \(0, n_features\]"),
-        ({"max_features": -0.1}, r"max_features must be in \(0, n_features\]"),
-        ({"n_iter_no_change": "invalid"}, "n_iter_no_change should either be"),
+        ({"learning_rate": 0}, ValueError, "learning_rate == 0, must be > 0.0"),
+        (
+            {"learning_rate": "foo"},
+            TypeError,
+            "learning_rate must be an instance of float",
+        ),
+        ({"n_estimators": 0}, ValueError, "n_estimators == 0, must be >= 1"),
+        (
+            {"n_estimators": 1.5},
+            TypeError,
+            "n_estimators must be an instance of int,",
+        ),
+        ({"loss": "foobar"}, ValueError, "Loss 'foobar' not supported"),
+        ({"subsample": 0.0}, ValueError, "subsample == 0.0, must be > 0.0"),
+        ({"subsample": 1.1}, ValueError, "subsample == 1.1, must be <= 1.0"),
+        (
+            {"subsample": "foo"},
+            TypeError,
+            "subsample must be an instance of float",
+        ),
+        ({"init": {}}, ValueError, "The init parameter must be an estimator or 'zero'"),
+        ({"max_features": 0}, ValueError, "max_features == 0, must be >= 1"),
+        ({"max_features": 0.0}, ValueError, "max_features == 0.0, must be > 0.0"),
+        ({"max_features": 1.1}, ValueError, "max_features == 1.1, must be <= 1.0"),
+        ({"max_features": "foobar"}, ValueError, "Invalid value for max_features."),
+        ({"verbose": -1}, ValueError, "verbose == -1, must be >= 0"),
+        (
+            {"verbose": "foo"},
+            TypeError,
+            "verbose must be an instance of",
+        ),
+        ({"warm_start": "foo"}, TypeError, "warm_start must be an instance of"),
+        (
+            {"validation_fraction": 0.0},
+            ValueError,
+            "validation_fraction == 0.0, must be > 0.0",
+        ),
+        (
+            {"validation_fraction": 1.0},
+            ValueError,
+            "validation_fraction == 1.0, must be < 1.0",
+        ),
+        (
+            {"validation_fraction": "foo"},
+            TypeError,
+            "validation_fraction must be an instance of float",
+        ),
+        ({"n_iter_no_change": 0}, ValueError, "n_iter_no_change == 0, must be >= 1"),
+        (
+            {"n_iter_no_change": 1.5},
+            TypeError,
+            "n_iter_no_change must be an instance of int,",
+        ),
+        ({"tol": 0.0}, ValueError, "tol == 0.0, must be > 0.0"),
+        (
+            {"tol": "foo"},
+            TypeError,
+            "tol must be an instance of float,",
+        ),
+        # The following parameters are checked in BaseDecisionTree
+        ({"min_samples_leaf": 0}, ValueError, "min_samples_leaf == 0, must be >= 1"),
+        ({"min_samples_leaf": 0.0}, ValueError, "min_samples_leaf == 0.0, must be > 0"),
+        (
+            {"min_samples_leaf": "foo"},
+            TypeError,
+            "min_samples_leaf must be an instance of float",
+        ),
+        ({"min_samples_split": 1}, ValueError, "min_samples_split == 1, must be >= 2"),
+        (
+            {"min_samples_split": 0.0},
+            ValueError,
+            "min_samples_split == 0.0, must be > 0.0",
+        ),
+        (
+            {"min_samples_split": 1.1},
+            ValueError,
+            "min_samples_split == 1.1, must be <= 1.0",
+        ),
+        (
+            {"min_samples_split": "foo"},
+            TypeError,
+            "min_samples_split must be an instance of float",
+        ),
+        (
+            {"min_weight_fraction_leaf": -1},
+            ValueError,
+            "min_weight_fraction_leaf == -1, must be >= 0.0",
+        ),
+        (
+            {"min_weight_fraction_leaf": 0.6},
+            ValueError,
+            "min_weight_fraction_leaf == 0.6, must be <= 0.5",
+        ),
+        (
+            {"min_weight_fraction_leaf": "foo"},
+            TypeError,
+            "min_weight_fraction_leaf must be an instance of float",
+        ),
+        ({"max_leaf_nodes": 0}, ValueError, "max_leaf_nodes == 0, must be >= 2"),
+        (
+            {"max_leaf_nodes": 1.5},
+            TypeError,
+            "max_leaf_nodes must be an instance of int",
+        ),
+        ({"max_depth": -1}, ValueError, "max_depth == -1, must be >= 1"),
+        (
+            {"max_depth": 1.1},
+            TypeError,
+            "max_depth must be an instance of int",
+        ),
+        (
+            {"min_impurity_decrease": -1},
+            ValueError,
+            "min_impurity_decrease == -1, must be >= 0.0",
+        ),
+        (
+            {"min_impurity_decrease": "foo"},
+            TypeError,
+            "min_impurity_decrease must be an instance of float",
+        ),
+        ({"ccp_alpha": -1.0}, ValueError, "ccp_alpha == -1.0, must be >= 0.0"),
+        (
+            {"ccp_alpha": "foo"},
+            TypeError,
+            "ccp_alpha must be an instance of float",
+        ),
+        ({"criterion": "mae"}, ValueError, "criterion='mae' is not supported."),
     ],
+    # Avoid long error messages in test names:
+    # https://github.com/scikit-learn/scikit-learn/issues/21362
+    ids=lambda x: x[:10].replace("]", "") if isinstance(x, str) else x,
 )
 @pytest.mark.parametrize(
     "GradientBoosting, X, y",
@@ -113,17 +219,20 @@ def test_classification_toy(loss):
         (GradientBoostingClassifier, iris.data, iris.target),
     ],
 )
-def test_gbdt_parameter_checks(GradientBoosting, X, y, params, err_msg):
+def test_gbdt_parameter_checks(GradientBoosting, X, y, params, err_type, err_msg):
     # Check input parameter validation for GradientBoosting
-    with pytest.raises(ValueError, match=err_msg):
-        GradientBoosting(**params).fit(X, y)
+    est = GradientBoosting(**params)
+    with pytest.raises(err_type, match=err_msg):
+        est.fit(X, y)
 
 
 @pytest.mark.parametrize(
     "params, err_msg",
     [
-        ({"loss": "huber", "alpha": 1.2}, r"alpha must be in \(0.0, 1.0\)"),
-        ({"loss": "quantile", "alpha": 1.2}, r"alpha must be in \(0.0, 1.0\)"),
+        ({"loss": "huber", "alpha": 0.0}, "alpha == 0.0, must be > 0.0"),
+        ({"loss": "quantile", "alpha": 0.0}, "alpha == 0.0, must be > 0.0"),
+        ({"loss": "huber", "alpha": 1.2}, "alpha == 1.2, must be < 1.0"),
+        ({"loss": "quantile", "alpha": 1.2}, "alpha == 1.2, must be < 1.0"),
     ],
 )
 def test_gbdt_loss_alpha_error(params, err_msg):
@@ -140,7 +249,7 @@ def test_gbdt_loss_alpha_error(params, err_msg):
         (GradientBoostingClassifier, "absolute_error"),
         (GradientBoostingClassifier, "quantile"),
         (GradientBoostingClassifier, "huber"),
-        (GradientBoostingRegressor, "deviance"),
+        (GradientBoostingRegressor, "log_loss"),
         (GradientBoostingRegressor, "exponential"),
     ],
 )
@@ -151,7 +260,7 @@ def test_wrong_type_loss_function(GradientBoosting, loss):
         GradientBoosting(loss=loss).fit(X, y)
 
 
-@pytest.mark.parametrize("loss", ("deviance", "exponential"))
+@pytest.mark.parametrize("loss", ("log_loss", "exponential"))
 def test_classification_synthetic(loss):
     # Test GradientBoostingClassifier on synthetic dataset used by
     # Hastie et al. in ESLII Example 12.7.
@@ -194,18 +303,22 @@ def test_regression_dataset(loss, subsample):
     ones = np.ones(len(y_reg))
     last_y_pred = None
     for sample_weight in [None, ones, 2 * ones]:
+        # learning_rate, max_depth and n_estimators were adjusted to get a mode
+        # that is accurate enough to reach a low MSE on the training set while
+        # keeping the resource used to execute this test low enough.
         reg = GradientBoostingRegressor(
-            n_estimators=100,
+            n_estimators=30,
             loss=loss,
             max_depth=4,
             subsample=subsample,
             min_samples_split=2,
             random_state=1,
+            learning_rate=0.5,
         )
 
         reg.fit(X_reg, y_reg, sample_weight=sample_weight)
         leaves = reg.apply(X_reg)
-        assert leaves.shape == (500, 100)
+        assert leaves.shape == (100, 30)
 
         y_pred = reg.predict(X_reg)
         mse = mean_squared_error(y_reg, y_pred)
@@ -230,7 +343,7 @@ def test_iris(subsample, sample_weight):
         sample_weight = np.ones(len(iris.target))
     # Check consistency on dataset iris.
     clf = GradientBoostingClassifier(
-        n_estimators=100, loss="deviance", random_state=1, subsample=subsample
+        n_estimators=100, loss="log_loss", random_state=1, subsample=subsample
     )
     clf.fit(iris.data, iris.target, sample_weight=sample_weight)
     score = clf.score(iris.data, iris.target)
@@ -362,8 +475,8 @@ def test_max_feature_regression():
         random_state=1,
     )
     gbrt.fit(X_train, y_train)
-    deviance = gbrt.loss_(y_test, gbrt.decision_function(X_test))
-    assert deviance < 0.5, "GB failed with deviance %.4f" % deviance
+    log_loss = gbrt._loss(y_test, gbrt.decision_function(X_test))
+    assert log_loss < 0.5, "GB failed with deviance %.4f" % log_loss
 
 
 def test_feature_importance_regression(fetch_california_housing_fxt):
@@ -398,6 +511,8 @@ def test_feature_importance_regression(fetch_california_housing_fxt):
     assert set(sorted_features[1:4]) == {"Longitude", "AveOccup", "Latitude"}
 
 
+# TODO(1.3): Remove warning filter
+@pytest.mark.filterwarnings("ignore:`max_features='auto'` has been deprecated in 1.1")
 def test_max_feature_auto():
     # Test if max features is set properly for floats and str.
     X, y = datasets.make_hastie_10_2(n_samples=12000, random_state=1)
@@ -644,7 +759,7 @@ def test_oob_improvement_raise():
 def test_oob_multilcass_iris():
     # Check OOB improvement on multi-class dataset.
     clf = GradientBoostingClassifier(
-        n_estimators=100, loss="deviance", random_state=1, subsample=0.5
+        n_estimators=100, loss="log_loss", random_state=1, subsample=0.5
     )
     clf.fit(iris.data, iris.target)
     score = clf.score(iris.data, iris.target)
@@ -984,15 +1099,18 @@ def test_complete_regression():
 
 
 def test_zero_estimator_reg():
-    # Test if init='zero' works for regression.
+    # Test if init='zero' works for regression by checking that it is better
+    # than a simple baseline.
 
+    baseline = DummyRegressor(strategy="mean").fit(X_reg, y_reg)
+    mse_baseline = mean_squared_error(baseline.predict(X_reg), y_reg)
     est = GradientBoostingRegressor(
-        n_estimators=20, max_depth=1, random_state=1, init="zero"
+        n_estimators=5, max_depth=1, random_state=1, init="zero", learning_rate=0.5
     )
     est.fit(X_reg, y_reg)
     y_pred = est.predict(X_reg)
-    mse = mean_squared_error(y_reg, y_pred)
-    assert_almost_equal(mse, 0.52, decimal=2)
+    mse_gbdt = mean_squared_error(y_reg, y_pred)
+    assert mse_gbdt < mse_baseline
 
     est = GradientBoostingRegressor(
         n_estimators=20, max_depth=1, random_state=1, init="foobar"
@@ -1108,7 +1226,7 @@ def test_non_uniform_weights_toy_edge_case_clf():
     y = [0, 0, 1, 0]
     # ignore the first 2 training samples by setting their weight to 0
     sample_weight = [0, 0, 1, 1]
-    for loss in ("deviance", "exponential"):
+    for loss in ("log_loss", "exponential"):
         gb = GradientBoostingClassifier(n_estimators=5, loss=loss)
         gb.fit(X, y, sample_weight=sample_weight)
         assert_array_equal(gb.predict([[1, 0]]), [1])
@@ -1159,50 +1277,61 @@ def test_sparse_input(EstimatorClass, sparse_matrix):
             assert_array_almost_equal(res_sparse, res)
 
 
-def test_gradient_boosting_early_stopping():
-    X, y = make_classification(n_samples=1000, random_state=0)
+@pytest.mark.parametrize(
+    "GradientBoostingEstimator", [GradientBoostingClassifier, GradientBoostingRegressor]
+)
+def test_gradient_boosting_early_stopping(GradientBoostingEstimator):
+    # Check if early stopping works as expected, that is empirically check that the
+    # number of trained estimators is increasing when the tolerance decreases.
 
-    gbc = GradientBoostingClassifier(
-        n_estimators=1000,
+    X, y = make_classification(n_samples=1000, random_state=0)
+    n_estimators = 1000
+
+    gb_large_tol = GradientBoostingEstimator(
+        n_estimators=n_estimators,
         n_iter_no_change=10,
         learning_rate=0.1,
         max_depth=3,
         random_state=42,
+        tol=1e-1,
     )
 
-    gbr = GradientBoostingRegressor(
-        n_estimators=1000,
+    gb_small_tol = GradientBoostingEstimator(
+        n_estimators=n_estimators,
         n_iter_no_change=10,
         learning_rate=0.1,
         max_depth=3,
         random_state=42,
+        tol=1e-3,
     )
 
     X_train, X_test, y_train, y_test = train_test_split(X, y, random_state=42)
-    # Check if early_stopping works as expected
-    for est, tol, early_stop_n_estimators in (
-        (gbc, 1e-1, 28),
-        (gbr, 1e-1, 13),
-        (gbc, 1e-3, 70),
-        (gbr, 1e-3, 28),
-    ):
-        est.set_params(tol=tol)
-        est.fit(X_train, y_train)
-        assert est.n_estimators_ == early_stop_n_estimators
-        assert est.score(X_test, y_test) > 0.7
+    gb_large_tol.fit(X_train, y_train)
+    gb_small_tol.fit(X_train, y_train)
+
+    assert gb_large_tol.n_estimators_ < gb_small_tol.n_estimators_ < n_estimators
+
+    assert gb_large_tol.score(X_test, y_test) > 0.7
+    assert gb_small_tol.score(X_test, y_test) > 0.7
+
+
+def test_gradient_boosting_without_early_stopping():
+    # When early stopping is not used, the number of trained estimators
+    # must be the one specified.
+    X, y = make_classification(n_samples=1000, random_state=0)
 
-    # Without early stopping
     gbc = GradientBoostingClassifier(
-        n_estimators=100, learning_rate=0.1, max_depth=3, random_state=42
+        n_estimators=50, learning_rate=0.1, max_depth=3, random_state=42
     )
     gbc.fit(X, y)
     gbr = GradientBoostingRegressor(
-        n_estimators=200, learning_rate=0.1, max_depth=3, random_state=42
+        n_estimators=30, learning_rate=0.1, max_depth=3, random_state=42
     )
     gbr.fit(X, y)
 
-    assert gbc.n_estimators_ == 100
-    assert gbr.n_estimators_ == 200
+    # The number of trained estimators must be the one specified.
+    assert gbc.n_estimators_ == 50
+    assert gbr.n_estimators_ == 30
 
 
 def test_gradient_boosting_validation_fraction():
@@ -1346,7 +1475,7 @@ def test_early_stopping_n_classes():
     X = [[1]] * 10
     y = [0, 0] + [1] * 8  # only 2 negative class over 10 samples
     gb = GradientBoostingClassifier(
-        n_iter_no_change=5, random_state=0, validation_fraction=8
+        n_iter_no_change=5, random_state=0, validation_fraction=0.8
     )
     with pytest.raises(
         ValueError, match="The training data after the early stopping split"
@@ -1355,7 +1484,7 @@ def test_early_stopping_n_classes():
 
     # No error if we let training data be big enough
     gb = GradientBoostingClassifier(
-        n_iter_no_change=5, random_state=0, validation_fraction=4
+        n_iter_no_change=5, random_state=0, validation_fraction=0.4
     )
 
 
@@ -1367,49 +1496,6 @@ def test_gbr_degenerate_feature_importances():
     assert_array_equal(gbr.feature_importances_, np.zeros(10, dtype=np.float64))
 
 
-# TODO: Remove in 1.1 when `n_classes_` is deprecated
-def test_gbr_deprecated_attr():
-    # check that accessing n_classes_ in GradientBoostingRegressor raises
-    # a deprecation warning
-    X = np.zeros((10, 10))
-    y = np.ones((10,))
-    gbr = GradientBoostingRegressor().fit(X, y)
-    msg = "Attribute `n_classes_` was deprecated"
-    with pytest.warns(FutureWarning, match=msg):
-        gbr.n_classes_
-
-
-# TODO: Remove in 1.1 when `n_classes_` is deprecated
-@pytest.mark.filterwarnings("ignore:Attribute `n_classes_` was deprecated")
-def test_attr_error_raised_if_not_fitted():
-    # check that accessing n_classes_ in not fitted GradientBoostingRegressor
-    # raises an AttributeError
-    gbr = GradientBoostingRegressor()
-    # test raise AttributeError if not fitted
-    msg = f"{GradientBoostingRegressor.__name__} object has no n_classes_ attribute."
-    with pytest.raises(AttributeError, match=msg):
-        gbr.n_classes_
-
-
-# TODO: Update in 1.1 to check for the error raised
-@pytest.mark.parametrize(
-    "estimator",
-    [
-        GradientBoostingClassifier(criterion="mae"),
-        GradientBoostingRegressor(criterion="mae"),
-    ],
-)
-def test_criterion_mae_deprecation(estimator):
-    # checks whether a deprecation warning is issues when criterion='mae'
-    # is used.
-    msg = (
-        "criterion='mae' was deprecated in version 0.24 and "
-        "will be removed in version 1.1"
-    )
-    with pytest.warns(FutureWarning, match=msg):
-        estimator.fit(X, y)
-
-
 # FIXME: remove in 1.2
 @pytest.mark.parametrize(
     "Estimator", [GradientBoostingClassifier, GradientBoostingRegressor]
@@ -1441,20 +1527,37 @@ def test_criterion_mse_deprecated(Estimator):
         assert_allclose(est1.predict(X), est2.predict(X))
 
 
-# TODO: Remove in v1.2
 @pytest.mark.parametrize(
-    "old_loss, new_loss",
+    "old_loss, new_loss, Estimator",
     [
-        ("ls", "squared_error"),
-        ("lad", "absolute_error"),
+        # TODO(1.2): Remove
+        ("ls", "squared_error", GradientBoostingRegressor),
+        ("lad", "absolute_error", GradientBoostingRegressor),
+        # TODO(1.3): Remove
+        ("deviance", "log_loss", GradientBoostingClassifier),
     ],
 )
-def test_loss_deprecated(old_loss, new_loss):
-    est1 = GradientBoostingRegressor(loss=old_loss, random_state=0)
+def test_loss_deprecated(old_loss, new_loss, Estimator):
+    est1 = Estimator(loss=old_loss, random_state=0)
 
-    with pytest.warns(FutureWarning, match=f"The loss '{old_loss}' was deprecated"):
+    with pytest.warns(FutureWarning, match=rf"The loss.* '{old_loss}' was deprecated"):
         est1.fit(X, y)
 
-    est2 = GradientBoostingRegressor(loss=new_loss, random_state=0)
+    est2 = Estimator(loss=new_loss, random_state=0)
     est2.fit(X, y)
     assert_allclose(est1.predict(X), est2.predict(X))
+
+
+# TODO(1.3): remove
+@pytest.mark.parametrize(
+    "Estimator", [GradientBoostingClassifier, GradientBoostingRegressor]
+)
+def test_loss_attribute_deprecation(Estimator):
+    # Check that we raise the proper deprecation warning if accessing
+    # `loss_`.
+    X = np.array([[1, 2], [3, 4]])
+    y = np.array([1, 0])
+    est = Estimator().fit(X, y)
+
+    with pytest.warns(FutureWarning, match="`loss_` was deprecated"):
+        est.loss_
